# @push.rocks/smartchok
smart wrapper for chokidar

## Install

Install the package by running the following command in your terminal:

```sh
npm install @push.rocks/smartchok --save
```

This command adds `@push.rocks/smartchok` to your project's dependencies, ensuring that your project can use its functionality and that it will be installed when running `npm install` in your project root.

## Usage

The `@push.rocks/smartchok` package provides a convenient and smart wrapper around the popular `chokidar` library (v4.x) for file watching with enhanced features such as observable support for filesystem events and glob pattern matching support. This guide will introduce you to the usage of `@push.rocks/smartchok`, leveraging TypeScript for type safety and better developer experience.

### Setting Up Your Project

To use `@push.rocks/smartchok`, ensure your project is set up to use TypeScript and ECMAScript modules (ESM). You need to have a `tsconfig.json` file at the root of your project with the following minimum settings:

```json
{
  "compilerOptions": {
    "target": "esnext",
    "module": "esnext",
    "moduleResolution": "node",
    "esModuleInterop": true,
    "declaration": true,
    "outDir": "./dist",
    "strict": true
  },
  "include": ["src/**/*"],
  "exclude": ["node_modules", "dist"]
}
```

This configuration enables TypeScript compilation targeting the latest ECMAScript standards and includes all TypeScript files (`*.ts`) in your `src` directory.

### Basic Setup

To start using `smartchok`, first import it into your TypeScript file:

```typescript
import { Smartchok } from '@push.rocks/smartchok';
```

#### Initializing Smartchok

Create an instance of `Smartchok` by specifying an array of glob patterns to watch:

```typescript
const smartchokInstance = new Smartchok([
  './src/**/*.ts',          // Watch all TypeScript files in the src directory
  './public/assets/**/*'    // Watch all files in the public/assets directory
]);
```

#### Adding and Removing Files Dynamically

You can dynamically add and remove paths from being watched by using the `add` and `remove` methods:

```typescript
// Add additional files or patterns
smartchokInstance.add(['./tests/**/*.spec.ts']);

// Remove specific patterns from watch
smartchokInstance.remove('./src/**/*.test.ts');
```

#### Handling Filesystem Events

`smartchok` leverages RxJS observables to notify about filesystem events. This allows you to respond to various events such as file additions, changes, and deletions with ease.

First, start the watcher:

```typescript
await smartchokInstance.start();
```

Then, subscribe to an event:

```typescript
const changeObservable = await smartchokInstance.getObservableFor('change');

changeObservable.subscribe({
  next: ([path, stats]) => {
    console.log(`File changed: ${path}`);
  },
  error: (err) => {
    console.error(`An error occurred: ${err}`);
  }
});
```

Supported events include 'add', 'change', 'unlink' (for deletions), 'addDir', 'unlinkDir', 'error', 'ready', and 'raw'. Since chokidar 4.x no longer supports glob patterns natively, smartchok handles glob pattern matching internally using picomatch.

#### Stopping the Watcher

To stop watching for file changes, simply call:

```typescript
await smartchokInstance.stop();
```

### Advanced Usage

Beyond the basics, `smartchok` allows for more complex monitoring scenarios, such as debounced notifications for rapid changes, filtering events, and integrating with other observables for complex asynchronous workflows.

### Conclusion

`@push.rocks/smartchok` provides a robust, observable-based wrapper around `chokidar` 4.x, making it an excellent choice for projects requiring efficient and flexible file monitoring. Its integration with RxJS opens up a wide array of possibilities for handling file system events in a reactive manner, making your code more concise, readable, and maintainable. The package maintains backward compatibility by implementing glob pattern support that was removed in chokidar 4.x.

By following the guidelines provided in this document, you should now be equipped to integrate `@push.rocks/smartchok` into your TypeScript project, enhancing its capabilities with efficient file system monitoring.

## License and Legal Information

This repository contains open-source code that is licensed under the MIT License. A copy of the MIT License can be found in the [license](license) file within this repository. 

**Please note:** The MIT License does not grant permission to use the trade names, trademarks, service marks, or product names of the project, except as required for reasonable and customary use in describing the origin of the work and reproducing the content of the NOTICE file.

### Trademarks

This project is owned and maintained by Task Venture Capital GmbH. The names and logos associated with Task Venture Capital GmbH and any related products or services are trademarks of Task Venture Capital GmbH and are not included within the scope of the MIT license granted herein. Use of these trademarks must comply with Task Venture Capital GmbH's Trademark Guidelines, and any usage must be approved in writing by Task Venture Capital GmbH.

### Company Information

Task Venture Capital GmbH  
Registered at District court Bremen HRB 35230 HB, Germany

For any legal inquiries or if you require further information, please contact us via email at hello@task.vc.

By using this repository, you acknowledge that you have read this section, agree to comply with its terms, and understand that the licensing of the code does not imply endorsement by Task Venture Capital GmbH of any derivative works.
